from .Twweeter import Twweeter
import os
import sys
from os.path import expanduser
from .Listener import Listener

# Twitter API credentials
home = expanduser("~")
OPTIONS = """
1. Get tweets of any user
2. Get tweets of particular hashtag
3. Get trending topics
4. Get your followers list
5. Get your tweets
Press 99 to exit or press 66 to go back to main menu ::
"""

HASHTAG_MSG = """
Enter the hashtag or word you may enter multiple words/hashtags separated by a "," : 
"""

def _decorator(f):
    f()
    print("DONE \n")
    return f


class TwweetCLI():

    def __init__(self):
        self.twweeter_obj = None
        self.listener_obj = None

    def _decorator(self, f):
        self.f()
        print("DONE \n")
        return self.f

    def check_data_dir_exists(self):
        try:
            original_umask = os.umask(0)
            os.makedirs(home + '/.twweet-cli/data')
        except OSError:
            pass
        finally:
            os.umask(original_umask)

    def home_select_action(self):

        option = input(OPTIONS)
        if option == '99':
            sys.exit(0)
        if option == '66':
            print(('\n\n'))
            return False
        if option == '1':
            self.twweeter_obj.get_all_tweets(
                input(
                    'Enter the username whose twweet\'s you want to grab '
                )
            )
        elif option == '2':
            words = input(HASHTAG_MSG)
            self.listener_obj.stream_word_or_hashtag(words_list=words)
        elif option == '3':
            self.twweeter_obj.get_trending_topics()
        elif option == '4':
            self.twweeter_obj.get_followers_list()
        elif option == '5':
            self.twweeter_obj.get_tweets()
        else:
            print(('Please choose any of the above options\n \n'))

    def main(self):
        self.twweeter_obj = Twweeter()
        self.listener_obj = Listener(self.twweeter_obj)
        self.check_data_dir_exists()
        print('Welcome to Twweet-Cli')
        print('Press 99 to quit the application')
        while True:
            option = input('Enter \'twweet\' or \'get\' or \'edit\': ')
            if option == '99':
                break
            if option == 'twweet':
                tweet = input('Enter your twweet\n')
                if len(tweet) >= 140:
                    print("Tweet length exceeds the limit of 140 characters!")
                    continue
                self.twweeter_obj.api.update_status(status=tweet)
                # Yes, tweet is called 'status' rather confusing
            elif option == 'get':
                check = True
                while check:
                    check = self.home_select_action()
            elif option == 'edit':
                self.twweeter_obj.create_creds()
            else:
                print('Please choose any of the above options \n \n')


def cli():
    tww = TwweetCLI()
    tww.main()


if __name__ == "__main__":
    cli()



import tweepy
import os
import sys
import csv
import json
from .config.ConfigReader import ConfigurationReader

# Twitter API credentials
home = os.path.expanduser("~")


class Twweeter():

    def __init__(self):
        self.cfg = self.get_creds()
        self.api = self.get_api(self.cfg)

    def get_api(self, cfg):
        # Twitter only allows access to a users most recent 3240 
        # tweets with this method
        """get api
        Args:
            cfg :
        Returns:
            Tweepy API
        """
        auth = tweepy.OAuthHandler(cfg['consumer_key'], cfg['consumer_secret'])
        auth.set_access_token(cfg['access_token'], cfg['access_token_secret'])
        return tweepy.API(auth)

    def get_all_tweets(self, screen_name):
        """get all tweets
        Args:
            screen_name :
        Returns:

        """
        # initialize a list to hold all the tweepy Tweets
        all_tweets = []

        # make initial request for most recent tweets (200 is the 
        # maximum allowed count)
        new_tweets = self.api.user_timeline(screen_name=screen_name, count=200)

        # save most recent tweets
        all_tweets.extend(new_tweets)

        # save the id of the oldest tweet less one
        oldest = all_tweets[-1].id - 1

        # keep grabbing tweets until there are no tweets left to grab
        # while respecting the api's rate limiting to avoid 429s
        # probably should lower this to be used a few times in a 15 
        # minute window

        last = 0
        for i in range(899):
            print(("getting tweets before {}".format(oldest)))

            # all subsiquent requests use the max_id param to prevent
            # duplicates
            new_tweets = self.api.user_timeline(screen_name=screen_name,
                                                count=200, max_id=oldest)

            # save most recent tweets
            all_tweets.extend(new_tweets)

            # update the id of the oldest tweet less one
            oldest = all_tweets[-1].id - 1

            print(("...{} tweets downloaded so far".format(len(all_tweets))))
            if last == len(all_tweets):
                break
            last = len(all_tweets)

        # transform the tweepy tweets into a 2D array that will
        # populate the csv
        out_tweets = [[tweet.id_str, tweet.created_at, tweet.text\
                        .encode("utf-8")]for tweet in all_tweets]

        configuration = ConfigurationReader()
        # write to csv
        tweets_storage = configuration.get_tweets_storage()
        tweets_storage = str(os.getcwd() + tweets_storage)
        os.makedirs(os.path.dirname(tweets_storage), exist_ok=True)
        f_name = str(tweets_storage + screen_name + '_tweets.csv')
        with open(f_name, 'w') as f:
            writer = csv.writer(f)
            writer.writerow(["id", "created_at", "text"])
            writer.writerows(out_tweets)

        pass

    # function to download the tweets of a particular hashtag
    def get_tweets_of_hashtag(self, hash_tag):
        print(("Please be patient while we download the tweets"))

        # initialize a list to hold all the tweepy Tweets
        all_tweets = []

        for i in range(179):
            new_tweets = tweepy.Cursor(self.api.search, q=hash_tag).items(200)
            for tweet in new_tweets:
                all_tweets.append(tweet.text.encode("utf-8"))
                # max_id will be id of last tweet when loop completes.
                # shitty way of doing things
                max_id = tweet.id

            print(("We have got {} tweets so far".format(len(all_tweets))))
            new_tweets = tweepy.Cursor(self.api.search, q=hash_tag).items(200)
            if (len(all_tweets)) >= 1000:
                break

        hashtag_storage = configuration.get_tweets_storage()
        hashtag_storage = str(os.getcwd() + hashtag_storage)
        os.makedirs(os.path.dirname(tweets_storage), exist_ok=True)
        file_name = str(hashtag_storage + hash_tag + 's.csv')
        with open(file_name, 'w') as f:
            writer = csv.writer(f)
            for tweet in all_tweets:
                if tweet:
                    writer.writerow([tweet])

        print(("1000 tweets have been saved to {}.csv".format(hash_tag)))

    def get_trending_topics(self):

        trends_1 = self.api.trends_place(1)  # 1 for worldwide
        data = trends_1[0]
        trends = data['trends']
        print("\nTrending topics worldwide :")
        for item in trends:
            print(item['name'])

    def process_or_store(self, tweet):
        print(json.dumps(tweet, indent=1))

    def read_timeline(self):
        # loop through the first ten items of your home timeline
        for status in tweepy.Cursor(self.api.home_timeline).items(10):
            # process a single status
            # print((status.text))
            self.process_or_store(status._json)

    def get_followers_list(self):
        id_num = 0
        num_followers = int(
            input('Enter number of followers you want to get: ')
        )
        for friend in tweepy.Cursor(self.api.followers).items(num_followers):
            friend = friend._json
            id_num += 1
            print("{}. {} -- @{}".format(id_num, friend["name"], \
                    friend["screen_name"]))

    def get_tweets(self):
        id_num = 0
        num_tweets = int(
            input('Enter the number of Tweets you want to get: ')
        )
        for tweet in tweepy.Cursor(self.api.user_timeline).items(num_tweets):
            # process_or_store(tweet._json)
            tweet = tweet._json
            id_num += 1
            print("{}.{}".format(id_num, tweet["text"]))

    def get_creds(self):
        if not os.path.isfile(home + '/.twweet-cli/data/creds.json'):
            self.create_creds()
        with open(home + '/.twweet-cli/data/creds.json') as json_file:
            return json.load(json_file)

    def create_creds(self):
        try:
            ck = input('Enter your Consumer Key: ').strip()
            cs = input('Enter your Consumer Secret: ').strip()
            at = input('Enter your Access Token: ').strip()
            ats = input('Enter your Access Token Secret: ').strip()
            jsondata = {"consumer_key": ck,
                        "consumer_secret": cs,
                        "access_token": at,
                        "access_token_secret": ats}

            with open(home + "/.twweet-cli/data/creds.json", "w") as outfile:
                json.dump(jsondata, outfile)

        except KeyError:
            sys.stderr.write("TWITTER_* environment variables not set\n")
            sys.exit(1)



import yaml
import os
from os.path import expanduser

home = expanduser("~")


class ConfigurationReader(object):
    __tweets = None
    __hashtag = None

    def __init__(self):
        ymlfile = home + '/.twweet-cli/data/config.yml'
        if os.path.isfile(ymlfile):
            with open(ymlfile, 'r') as file:
                cfg = yaml.load(file)
            ConfigurationReader.__tweets = cfg['Tweets']
            ConfigurationReader.__hashtag = cfg['HashTag']
        else:
            if not os.path.exists(home + '/.twweet-cli/data'):
                os.makedirs(home + '/.twweet-cli/data')

            with open(home + '/.twweet-cli/data/config.yml', 'w') as ymlfile:
                ymlLine1 = "#Depending on the system(Windows or Linux) change the\
                                 backward or forward slash appropriately."
                ymlLine2 = "Tweets: /TweetsStore/"
                ymlLine3 = "HashTag: /HashTagStore/"
                ymlfile.write("%s\n%s\n%s\n" % (ymlLine1, ymlLine2, ymlLine3))

    @staticmethod
    def get_tweets_storage():
        return ConfigurationReader.__tweets

    @staticmethod
    def get_hashtag_storage():
        return ConfigurationReader.__hashtag



import tweepy

'''STREAM'''


class StreamListener(tweepy.StreamListener):
    # Decided I would keep all the overridable functions from the 
    # BaseClass so we know what we have to play with.
    def __init__(self, twweeter_obj, time_limit=60):
        self.twweeter_obj = twweeter_obj
        super(StreamListener, self).__init__(self.twweeter_obj.api)

    def on_status(self, status):
        print('@{} => {}'.format(status.user.screen_name,
                                 status.text.replace("\n", " ")))

    def on_error(self, status_code):
        print('AN ERROR: {}'.format(status_code))
    #   read the docs and handle different errors

    def keep_alive(self):
        """Called when a keep-alive arrived"""
        return

    def on_exception(self, exception):
        """Called when an unhandled exception occurs."""
        return

    def on_delete(self, status_id, user_id):
        """Called when a delete notice arrives for a status"""
        return

    def on_event(self, status):
        """Called when a new event arrives"""
        return

    def on_direct_message(self, status):
        """Called when a new direct message arrives"""
        return

    def on_friends(self, friends):
        """Called when a friends list arrives.

        friends is a list that contains user_id
        """
        return

    def on_limit(self, track):
        """Called when a limitation notice arrives"""
        return

    def on_timeout(self):
        """Called when stream connection times out"""
        return

    def on_disconnect(self, notice):
        """Called when twitter sends a disconnect notice

        Disconnect codes are listed here:
        https://dev.twitter.com/docs/streaming-apis/\
        messages#Disconnect_messages_disconnect
        """
        return

    def on_warning(self, notice):
        """Called when a disconnection warning message arrives"""
        return


class Listener():

    def __init__(self, twweeter_obj):
        self.twweeter_obj = twweeter_obj
        self.stream_listener_ob = StreamListener(self.twweeter_obj)

    # authorise the StreamListener
    def auth_streamer(self):
        stream = tweepy.Stream(auth=self.twweeter_obj.api.auth,
                               listener=self.stream_listener_ob)
        return stream

    # listen for tweets containing a specific word or hashtag \
    #(a phrase might work too)
    def stream_word_or_hashtag(self, words_list):
        stream = self.auth_streamer()
        stream.filter(track=words_list, is_async=True)


'''END STREAM'''



"""A setuptools based setup module.
See:
https://packaging.python.org/en/latest/distributing.html
https://github.com/pypa/sampleproject
"""

from setuptools.command.install import install
from setuptools import setup, find_packages
from codecs import open
import os
import json
import shutil

from dotenv import load_dotenv
load_dotenv()

here = os.path.abspath(os.path.dirname(__file__))
home = os.path.expanduser("~")

with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


class TwtApiDetails(install):
    def run(self):
        print("Creating Data Directories...")
        check_data_dir_exists()
        create_creds()
        print("""Creating default config files.
              Please note that this overwrites the files if exist....""")
        with open(home + '/.twweet-cli/data/config.yml', 'w') as ymlfile:
            ymlLine1 = "#Depending on the system(Windows or Linux) change the\
                             backward or forward slash appropriately."
            ymlLine2 = "Tweets: /TweetsStore/"
            ymlLine3 = "HashTag: /HashTagStore/"
            ymlfile.write("%s\n%s\n%s\n" % (ymlLine1, ymlLine2, ymlLine3))
            install.run(self)


def check_data_dir_exists():
    try:
        original_umask = os.umask(0)
        if os.path.exists(home + '/.twweet-cli/data'):
            response = input("""The data directory already exists.
                             Would you like to overwrite? [yes/no] : """)
            if response.lower() == 'yes':
                shutil.rmtree(home + '/.twweet-cli/data')
                os.makedirs(home + '/.twweet-cli/data')
            elif response.lower() == 'no':
                print("Not overwriting...")
    except OSError:
        print("Cannot create data dir the installation cannot continue..")
        exit()
    finally:
        os.umask(original_umask)


def create_creds():
    try:
        jsondata = {
            "consumer_key": os.environ["CONSUMER_KEY"].strip(),
            "consumer_secret": os.environ["CONSUMER_SECRET"].strip(),
            "access_token": os.environ["ACCESS_TOKEN"].strip(),
            "access_token_secret": os.environ["ACCESS_TOKEN_SECRET"].strip()
        }
    except Exception as e:
        print("Make sure to set your Twitter OAuth credentials in your .env\
                 file. Refer to Readme for project setup.")
        raise e
    with open(home + "/.twweet-cli/data/creds.json", "w") as outfile:
        json.dump(jsondata, outfile)
    os.chmod(home + '/.twweet-cli/data/creds.json', 0o777)


setup(
    name='twweet_cli',
    version='2.0.6',
    description='Tweet right from your cli without even opening your browser.',
    long_description=long_description,
    url='https://github.com/CruiseDevice/twweet-cli/',
    author='Akash Chavan',
    author_email='achavan1211@gmail.com',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.6',
    ],
    cmdclass={
        'install': TwtApiDetails,
    },
    python_requires='>=3.6',
    keywords='twitter tweet cli',
    packages=find_packages(),
    install_requires=[
        'tweepy',
        'pyyaml',
        'python-dotenv',
    ],
    entry_points={
        'console_scripts': [
            'twweet-cli = twweet_cli.main:cli',
        ],
    },
)
